@@ -1,6 +1,6 @@
 /*
  * Stellarium
- * Copyright (C) 2002 Fabien Chreau
+ * Copyright (C) 2002 Fabien Chereau
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,7 +27,7 @@ void moy(Vec3f * p, int a, int b, int c)
 	p[c].normalize();
 }
 
-Grid::Grid()
+Grid::Grid() : result(NULL)
 {
 
 	NbPoints = 492;
@@ -692,25 +692,71 @@ Grid::Grid()
 {0.6554,0.1490,-0.740  },
 {0.6554,-0.1490,-0.740 }};
 */
+
+/*float tempTab42[42][3] ={
+{0.0000,0.0000,1.0000   },
+{0.8944,0.0000,0.4472   },
+{0.2764,0.8507,0.4472   },
+{-0.7236,0.5257,0.4472  },
+{-0.7236,-0.5257,0.4472 },
+{0.2764,-0.8507,0.4472  },
+{0.7236,0.5257,-0.4472  },
+{-0.2764,0.8507,-0.4472 },
+{-0.8944,-0.0000,-0.4472},
+{-0.2764,-0.8507,-0.4472},
+{0.7236,-0.5257,-0.4472 },
+{0.0000,0.0000,-1.0000  },
+{0.5257,0.0000,0.8507   },
+{0.1625,0.5000,0.8507   },
+{-0.4253,0.3090,0.8507  },
+{-0.4253,-0.3090,0.8507 },
+{0.1625,-0.5000,0.8507  },
+{0.6882,0.5000,0.5257   },
+{-0.2629,0.8090,0.5257  },
+{-0.8507,-0.0000,0.5257 },
+{-0.2629,-0.8090,0.5257 },
+{0.6882,-0.5000,0.5257  },
+{0.9511,0.3090,0.0000   },
+{-0.0000,1.0000,0.0000  },
+{-0.9511,0.3090,0.0000  },
+{-0.5878,-0.8090,0.0000 },
+{0.5878,-0.8090,0.0000  },
+{0.9511,-0.3090,0.0000  },
+{0.5878,0.8090,0.0000   },
+{-0.5878,0.8090,0.0000  },
+{-0.9511,-0.3090,0.0000 },
+{0.0000,-1.0000,0.0000  },
+{0.2629,0.8090,-0.5257  },
+{-0.6882,0.5000,-0.5257 },
+{-0.6882,-0.5000,-0.5257},
+{0.2629,-0.8090,-0.5257 },
+{0.8507,0.0000,-0.5257  },
+{0.4253,0.3090,-0.8507  },
+{-0.1625,0.5000,-0.8507 },
+{-0.5257,-0.0000,-0.8507},
+{-0.1625,-0.5000,-0.8507},
+{0.4253,-0.3090,-0.8507 }};*/
+
 	Points = new Vec3f[NbPoints];
 	for (int i=0;i<NbPoints;i++)
 	{
 		Points[i]=tempTab[i];
 	}
+	
+	result = new int[NbPoints];
 }
 
 Grid::~Grid()
 {
-    delete Points;
+    delete [] Points;
+	Points = NULL;
+	delete [] result; result=NULL;
 }
 
-int Grid::GetNearest(Vec3f v)
+int Grid::GetNearest(Vec3f& v)
 {
-    static int bestI;
-    static float bestDot;
-
-	bestI = -1;
-	bestDot = -2.;
+	int bestI = -1;
+	float bestDot = -2.f;
 
 	v.normalize();
     for(int i=0;i<NbPoints;++i)
@@ -733,10 +779,8 @@ void Grid::Draw(void)
 }
 
 // Return an array with the number of the zones in the field of view
-int Grid::Intersect(Vec3f pos, float fieldAngle, int * &result)
+int Grid::Intersect(Vec3f pos, float fieldAngle)
 {
-	if (result==NULL) result = new int[NbPoints];
-
 	pos.normalize();
 	float max = cosf(fieldAngle/2.f + Angle);
 
@@ -746,7 +790,7 @@ int Grid::Intersect(Vec3f pos, float fieldAngle, int * &result)
 		if(pos.dot(Points[i]) > max)
 		{
 			result[nbResult]=i;
-			nbResult++;
+			++nbResult;
 		}
 	}
 	return nbResult;
