@@ -1,6 +1,6 @@
 /*
  * Stellarium
- * Copyright (C) 2002 Fabien Chreau
+ * Copyright (C) 2002 Fabien Chereau
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,12 +19,18 @@
 
 #include <string>
 #include <iostream>
+#include <cstdlib>
+
 #include "SDL.h"
 
 #include "stellarium.h"
 #include "stel_core.h"
 #include "stel_sdl.h"
 
+#if defined(MACOSX) && defined(XCODE)
+#include "StelConfig.h"
+#endif
+
 using namespace std;
 
 string DDIR;	// Data Directory
@@ -42,19 +48,20 @@ void drawIntro(void)
     printf("    |##   #  ### ### ### # # # #  #  ### # #|\n");
     printf("    |                       %s|\n",APP_NAME);
     printf("    -----------------------------------------\n\n");
-    printf("Copyright (C) 2004 Fabien Chereau\n\n");
-    printf("Please check last version and send bug report & comments \n");
-    printf("on stellarium web page : http://stellarium.free.fr\n\n");
+    printf("Copyright (C) 2000-2005 Fabien Chereau et al.\n\n");
+    printf(_("Please check last version and send bug report & comments \n\
+on stellarium web page : http://www.stellarium.org\n\n"));
 };
 
 
 // Display stellarium usage in the console
 void usage(char **argv)
 {
-	printf("Usage: %s [OPTION] ...\n -v, --version          output version information and exit\n -h, --help             display this help and exit\n", argv[0]);
+	printf(_("Usage: %s [OPTION] ...\n -v, --version          Output version information and exit.\n -h, --help             Display this help and exit.\n"), argv[0]);
 }
 
 
+
 // Check command line arguments
 void check_command_line(int argc, char **argv)
 {
@@ -74,8 +81,8 @@ void check_command_line(int argc, char **argv)
 
     if (argc > 1)
     {
-        printf("%s: Bad command line argument(s)\n", argv[0]);
-        printf("Try `%s --help' for more information.\n", argv[0]);
+        printf(_("%s: Bad command line argument(s)\n"), argv[0]);
+        printf(_("Try `%s --help' for more information.\n"), argv[0]);
         exit(1);
     }
 }
@@ -86,6 +93,7 @@ void check_command_line(int argc, char **argv)
 // This enable to launch stellarium from the local directory without installing it
 void setDirectories(const char* executableName)
 {
+#if !defined(MACOSX) && !defined(XCODE)
 	// The variable CONFIG_DATA_DIR must have been set by the configure script
 	// Its value is the dataRoot directory, ie the one containing data/ and textures/
 
@@ -104,6 +112,7 @@ void setDirectories(const char* executableName)
 		if (tempFile)
 		{
 			DATA_ROOT = ".";
+			
 		}
 		else
 		{
@@ -115,10 +124,10 @@ void setDirectories(const char* executableName)
 			else
 			{
             	// Failure....
-            	printf("ERROR : I can't find the datas directories in :\n");
-            	printf("%s/ nor in ./ nor in ../\n",CONFIG_DATA_DIR);
-                printf("You may fully install the software (typ \"make install\" on POSIX systems)\n");
-                printf("or launch the application from the stellarium package directory.\n");
+            	printf(_("ERROR : I can't find the datas directories in :\n\
+%s/ nor in ./ nor in ../\n\
+You may fully install the software (type \"make install\" on POSIX systems)\n\
+or launch the application from the stellarium package directory.\n"),CONFIG_DATA_DIR);
                 exit(-1);
 			}
 		}
@@ -158,7 +167,7 @@ void setDirectories(const char* executableName)
 
 	// Just an indication if we are on unix/linux that we use local data files
 	if (DATA_ROOT != string(CONFIG_DATA_DIR))
-		cout << ">Found data files in " << DATA_ROOT << " : local version." << endl;
+		printf(_("> Found data files in %s : local version.\n"), DATA_ROOT.c_str());
 
 	// The problem is more complexe in the case of a unix/linux system
 	// The config files are in the HOME/.stellarium/ directory and this directory
@@ -176,7 +185,7 @@ void setDirectories(const char* executableName)
 	}
 	else
 	{
-		cout << "Will create config files in " << CDIR << endl;
+		printf(_("Will create config files in %s\n"), CDIR.c_str());
 		if ((tempFile = fopen((CDIR + "config.ini").c_str(),"w")))
 		{
 			fclose(tempFile);
@@ -184,7 +193,7 @@ void setDirectories(const char* executableName)
 		else
 		{
 			// Maybe the directory is not created so try to create it
-			cout << "Try to create directory " << CDIR << endl;
+			printf(_("Try to create directory %s\n"),CDIR.c_str());
 			system(string("mkdir " + CDIR).c_str());
 
 			if ((tempFile = fopen((CDIR + "config.ini").c_str(),"w")))
@@ -193,9 +202,9 @@ void setDirectories(const char* executableName)
 			}
 			else
 			{
-				cout << "Can't create the file " << CDIR << "config.ini" << endl;
-				cout << "If the directory " << CDIR << " is missing please create it by hand." << endl;
-				cout << "If not check that you have access to " << CDIR << endl;
+				printf(_("Can't create the file %sconfig.ini\n\
+If the directory %s is missing please create it by hand.\n\
+If not, check that you have access to %s\n"), CDIR.c_str(), CDIR.c_str(), CDIR.c_str());
 				exit(-1);
 			}
 		}
@@ -205,6 +214,20 @@ void setDirectories(const char* executableName)
 	}
 #endif	// Unix system
 
+#else   // Mac OS X
+    const char *ddir = NULL;
+    const char *tdir = NULL;
+    const char *cdir = NULL;
+    const char *data_root = NULL;
+
+    if (!setDirectories(&ddir, &tdir, &cdir, &data_root))
+        exit(-1);
+
+    DDIR = ddir;
+    TDIR = tdir;
+    CDIR = cdir;
+    DATA_ROOT = data_root;
+#endif
 }
 
 
@@ -212,6 +235,22 @@ void setDirectories(const char* executableName)
 // Main stellarium procedure
 int main(int argc, char **argv)
 {
+
+//Tony - added "&& !defined(WIN32)"
+#if !defined(MACOSX) && !defined(WIN32)
+	// TESTING ONLY
+	// setenv("LC_ALL", "fr_FR", 1);
+
+	setlocale (LC_CTYPE, "");
+	setlocale (LC_MESSAGES, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
+	// TODO: move to utf8
+	bind_textdomain_codeset(PACKAGE, "iso-8859-1");
+
+#endif
+	
 	// Check the command line
 	check_command_line(argc, argv);
 
@@ -222,12 +261,14 @@ int main(int argc, char **argv)
     setDirectories(argv[0]);
 
 	// Create the core of stellarium, it has to be initialized
-	stel_core* core = new stel_core();
-
-	core->set_directories(DDIR, TDIR, CDIR, DATA_ROOT);
+	stel_core* core = new stel_core(DDIR, TDIR, CDIR, DATA_ROOT);
 
 	// Give the config file parameters which has to be given "hard coded"
+#if !defined(MACOSX) && !defined(XCODE)
 	core->set_config_files("config.ini");
+#else
+    core->set_config_files(STELLARIUM_CONF_FILE);
+#endif
 
 	// Load the configuration options from the given file names
 	// This includes the video parameters
@@ -241,6 +282,12 @@ int main(int argc, char **argv)
 
 	core->init();
 
+    // Tony - added Re-load of config to re-enable flags available once the core has loaded 
+	core->load_config();
+
+	// play startup script, if available
+	core->play_startup_script();
+
 	// Start the main loop until the end of the execution
 	sdl_mgr.start_main_loop();
 
