@@ -1,6 +1,6 @@
 /*
 * Stellarium
-* Copyright (C) 2002 Fabien Chreau
+* Copyright (C) 2002 Fabien Chereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -20,8 +20,10 @@
 #include <iostream>
 #include <sstream>
 #include <fstream>
-
+// Tony - for editbox blink timer
+#include "SDL_timer.h"
 #include "s_gui.h"
+#include "stel_utility.h"
 
 using namespace s_gui;
 
@@ -166,7 +168,10 @@ Painter::Painter(const s_texture* _tex1, const s_font* _font,
 		baseColor(_baseColor),
 		textColor(_textColor)
 {
+}
 
+Painter::~Painter()
+{
 }
 
 // Draw the edges of the defined square with the default base color
@@ -300,7 +305,7 @@ void Painter::drawLine(const s_vec2i& pos1, const s_vec2i& pos2, const s_color&
 // Mother class for every s_gui object.
 ////////////////////////////////////////////////////////////////////////////////
 
-Component::Component(void) :
+Component::Component() :
 	pos(0, 0),
 	size(0, 0),
 	visible(1),
@@ -337,6 +342,11 @@ void Component::initScissor(int winW, int winH)
 	scissor = new Scissor(winW, winH);
 }
 
+void Component::deleteScissor(void)
+{
+	if (scissor) delete scissor;
+	scissor = NULL;
+}
 
 CallbackComponent::CallbackComponent() : Component(), is_mouse_over(0)
 {
@@ -380,7 +390,7 @@ int CallbackComponent::onMove(int x, int y)
 }
 
 //////////////////////////////// Container /////////////////////////////////////
-// Manages hierarchical components : send signals ad actions  to childrens
+// Manages hierarchical components : send signals and actions to childrens
 ////////////////////////////////////////////////////////////////////////////////
 
 Container::Container() : CallbackComponent()
@@ -392,11 +402,8 @@ Container::~Container()
     list<Component*>::iterator iter = childs.begin();
     while (iter != childs.end())
     {
-        if (*iter)
-        {
-            delete (*iter);
-            (*iter)=NULL;
-        }
+        delete (*iter);
+        (*iter)=NULL;
         iter++;
     }
 }
@@ -473,7 +480,7 @@ int Container::onMove(int x, int y)
 	return CallbackComponent::onMove(x, y);
 }
 
-int Container::onKey(SDLKey k, S_GUI_VALUE s)
+int Container::onKey(Uint16 k, S_GUI_VALUE s)
 {
 	if (!visible) return 0;
 	list<Component*>::iterator iter = childs.begin();
@@ -519,7 +526,7 @@ int Button::onClic(int x, int y, S_GUI_VALUE bt, S_GUI_VALUE state)
 {
 	if (!visible) return 0;
 	CallbackComponent::onClic(x,y,bt,state);
-	if (state==S_GUI_PRESSED && bt==S_GUI_MOUSE_LEFT && isIn(x, y))	return 1;
+	if (state==S_GUI_PRESSED && bt==S_GUI_MOUSE_LEFT && isIn(x, y)) return 1;
 	return 0;
 }
 
@@ -636,13 +643,21 @@ void Label::setLabel(const string& _label)
 void Label::draw()
 {
 	if (!visible) return;
-    glPushMatrix();
     if (painter.getFont())
     {
 		painter.print(pos[0], pos[1], label);
     }
-    glPopMatrix();
-	//painter.drawSquareEdge(pos, size);
+	// painter.drawSquareEdge(pos, size);
+}
+
+void Label::draw(float _intensity)
+{
+	if (!visible) return;
+    if (painter.getFont())
+    {
+		painter.print(pos[0], pos[1], label, painter.getTextColor() * _intensity);
+    }
+	// painter.drawSquareEdge(pos, size);
 }
 
 void Label::adjustSize(void)
@@ -652,6 +667,330 @@ void Label::adjustSize(void)
 }
 
 
+/////////////////////////////// EditBox //////////////////////////////////
+// Editbox - Tony
+////////////////////////////////////////////////////////////////////////////////
+
+EditBox *focusEditBox = NULL;
+bool blinkOn;
+
+EditBox::EditBox(const string& _label, const s_font* font) : Button(), label(_label, font), isEditing(false)
+{
+	Component::setSize(label.getSize()+s_vec2i(4,2));
+	text = _label;
+    cursorPos = 0;
+    resetHistory();
+    blinkOn = false;
+    autoCompleteOptions = "";
+    countAutoCompleteOptions = 0;
+    countAutoCompletePos = 0;
+    autoCompleteReady = false;
+}
+
+EditBox::~EditBox()
+{
+    SDL_SetTimer(0, NULL);
+    focusEditBox = NULL;       
+}
+
+int EditBox::onClic(int x, int y, S_GUI_VALUE bt, S_GUI_VALUE state)
+{
+	if (!visible) return 0;
+	if (state==S_GUI_PRESSED && bt==S_GUI_MOUSE_LEFT)
+	{
+		if (isIn(x, y)) 
+        {
+            setFocus();
+            return 1;
+        }
+		else if (isEditing) 
+		{
+            resetFocus();
+            return 1;
+         }
+	}
+	return Button::onClic(x,y,bt,state);
+}
+
+void EditBox::draw(void)
+{
+    if (!visible) return;
+	Button::draw();
+    glPushMatrix();
+    glTranslatef(pos[0], pos[1], 0.f);
+    Component::scissor->push(pos, size);
+	label.setPos(6, (size[1]-label.getSizey())/2+1);
+
+    refreshLabel();
+	if (isEditing) label.draw();
+    else label.draw(.3);   // faded
+    
+    Component::scissor->pop();
+	glPopMatrix();
+}
+
+#define EDITBOX_CURSOR "\7"
+void EditBox::refreshLabel(void)
+{
+     if (!isEditing) label.setLabel("> " + text);
+     else
+     {
+        if (!blinkOn) label.setLabel("> " + text); 
+        else
+        {
+            if (cursorPos == text.length()) // at end
+               label.setLabel("> " + text + EDITBOX_CURSOR); 
+            else
+               label.setLabel("> " + text.substr(0,cursorPos) + EDITBOX_CURSOR + text.substr(cursorPos)); 
+        }
+     }
+}
+
+void EditBox::clearText(void)
+{
+     cursorPos = 0;
+     text = "";             
+}
+
+Uint32 toggleBlink(Uint32 interval)
+{
+     SDL_SetTimer(0, NULL);
+     if (blinkOn == true)
+     {
+        blinkOn = false;
+        SDL_SetTimer(400, (SDL_TimerCallback)toggleBlink);
+     }
+     else
+     {
+        blinkOn = true;
+        SDL_SetTimer(600, (SDL_TimerCallback)toggleBlink);
+    }
+     
+     return interval;
+}
+
+void EditBox::setFocus(void)
+{
+      isEditing = true;
+      blinkOn = true;
+      resetAutoComplete(false);
+      SDL_SetTimer(600, (SDL_TimerCallback)toggleBlink);
+}
+
+void EditBox::resetFocus(void)
+{
+      isEditing = false;
+      blinkOn = false;
+      SDL_SetTimer(0, NULL);
+      focusEditBox = NULL;
+      resetAutoComplete(false);
+}
+
+int EditBox::onKey(Uint16 k, S_GUI_VALUE s)
+{
+    if (!isEditing) return 0;
+
+	if (s==S_GUI_PRESSED)
+    { 
+        if  (k==SDLK_RETURN)
+        {
+            resetAutoComplete(true);
+            addHistory(text);
+       		if (!onReturnKeyCallback.empty()) onReturnKeyCallback();
+       		clearText();
+       		resetFocus();
+            return 1;
+        }
+
+  		if (k == SDLK_TAB) resetAutoComplete(true);
+  		else if (k == SDLK_UP)
+  		{
+              text = prevHistory();
+              cursorPos = text.length();
+        }
+  		else if (k == SDLK_DOWN)
+  		{
+              text = nextHistory();
+              cursorPos = text.length();
+        }
+  		else if (k == SDLK_LEFT)  		
+  		{
+            if (SDL_GetModState() & KMOD_CTRL)
+                cursorToPrevWord();
+            else
+                if (cursorPos > 0) cursorPos--;
+        }
+        else if (k == SDLK_RIGHT)
+        {
+            if (SDL_GetModState() & KMOD_CTRL)
+                cursorToNextWord();
+            else
+              if (cursorPos < text.length()) cursorPos++;
+        }
+        else if (k == SDLK_HOME) cursorPos = 0;
+        else if (k == SDLK_END)  cursorPos = text.length();
+  		else if (k == SDLK_DELETE)
+  		{
+           resetAutoComplete(false);
+           if (cursorPos < text.length()) text = text.erase(cursorPos, 1);
+        }
+        else if (k == SDLK_BACKSPACE)
+        {
+           resetAutoComplete(false);
+           if (cursorPos > 0)
+           {
+               cursorPos--;
+               text = text.erase(cursorPos, 1);
+           }
+        }
+        else if (k == SDLK_ESCAPE) clearText(); 
+        else if ((k >= SDLK_0 && k <= SDLK_9) || (k >= SDLK_a && k <= SDLK_z) 
+        || (k >= 65 && k <= 90) || (k >= 224 && k <= 255) || k == SDLK_SPACE 
+				 || k == SDLK_UNDERSCORE || k == 39 || k == 45)
+        {
+            resetAutoComplete(false);
+            string newtext = "";
+            newtext += text.substr(0, cursorPos);
+            newtext += k;
+            newtext += text.substr(cursorPos);
+            text = newtext;
+            cursorPos++;
+        }
+        else
+            return 0;
+
+       if (!text.empty()) testAutoComplete();
+        return 1;
+    }
+
+    return 0;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// EditBox::History
+
+void EditBox::resetHistory(void)
+{
+    historyPos = 0;
+    historyMaxPos = 0;
+    for (int i = 0; i < 9; i++) history[i] = "";
+}
+
+void EditBox::addHistory(const string& _history)
+{
+     history[historyPos] = _history;
+
+     if (++historyMaxPos > 9) historyMaxPos = 9;
+     if (++historyPos > 9) historyPos = 0;
+}       
+
+string EditBox::prevHistory(void)
+{
+     if (--historyPos < 0) historyPos = historyMaxPos;
+     return history[historyPos];
+}
+
+string EditBox::nextHistory(void)
+{
+     if (++historyPos > 9) historyPos = 0;
+     return history[historyPos];
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// EditBox::AutoCompete
+
+void EditBox::testAutoComplete(void)
+{
+     if (!lstAutoComplete.empty())
+     {
+         autoCompleteOptions = "";
+         countAutoCompleteOptions = 0;
+         countAutoCompletePos = 0;
+         autoCompleteReady = false;
+         
+         unsigned int i = 0;
+         while (i < lstAutoComplete.size())
+         {
+               if (fcompare(lstAutoComplete[i], text) == 0)
+               {
+                 if (autoCompleteOptions.empty())
+                 { 
+                    autoCompleteOptions += lstAutoComplete[i];
+                    firstAutoComplete = lstAutoComplete[i];
+                 }
+                 else autoCompleteOptions = autoCompleteOptions + ", " + lstAutoComplete[i];
+                 if (countAutoCompleteOptions++ > 5)
+                   break;
+               }
+               i++;
+         }
+         
+         if (!autoCompleteOptions.empty()) 
+         {
+            countAutoCompletePos = text.length();
+             text = firstAutoComplete;
+             cursorPos = countAutoCompletePos;
+             autoCompleteReady = true;
+         }
+         if (!onAutoCompleteCallback.empty()) onAutoCompleteCallback();
+     }
+}
+
+void EditBox::resetAutoComplete(bool keepChanges)
+{
+     if (autoCompleteReady && !firstAutoComplete.empty() && !keepChanges)
+     { 
+         clearText();
+         text = firstAutoComplete.substr(0,countAutoCompletePos);
+         cursorPos = countAutoCompletePos;
+     }
+     else
+     {
+         if (!firstAutoComplete.empty()) cursorPos = text.length();
+     }
+
+      autoCompleteOptions = "";
+      countAutoCompleteOptions = 0;
+      countAutoCompletePos = 0;
+      autoCompleteReady = false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// EditBox::Cursor Previous and next words
+
+void EditBox::cursorToNextWord(void)
+{
+    while (cursorPos < text.length())
+    {
+        cursorPos++;  
+        if (text[cursorPos] == ' ')
+        {
+            if (cursorPos + 1 < text.length() && text[cursorPos+1] != ' ')
+            {
+               cursorPos++;
+               break;
+               }
+        }
+    }
+}
+
+void EditBox::cursorToPrevWord(void)
+{
+    while (cursorPos > 0)
+    {
+        cursorPos--;
+        // cater for the cursor at the start of the word already 
+        while (cursorPos > 0 && text[cursorPos] == ' ' && text[cursorPos-1] == ' ')
+              cursorPos--;
+
+        if (text[cursorPos] == ' ' && text[cursorPos+1] != ' ')
+        {
+            cursorPos++;
+            break;
+        }
+    }
+}
+
 /////////////////////////////// LabeledButton //////////////////////////////////
 // Button with text on it
 ////////////////////////////////////////////////////////////////////////////////
@@ -855,6 +1194,7 @@ int StdWin::onMove(int x, int y)
 	{
 		pos+=(s_vec2i(x,y)-oldpos);
 		oldpos.set(x,y);
+		return 1;
 	}
 	return 0;
 }
@@ -882,6 +1222,35 @@ void StdBtWin::onHideBt(void)
 }
 
 
+StdTransBtWin::StdTransBtWin(const char * _title, int _time_out, s_texture* _header_tex, s_font * _winfont, int headerSize) :
+	StdBtWin(_title, _header_tex, _winfont, headerSize)
+{
+
+	set_timeout(_time_out);
+}
+
+void StdTransBtWin::update(int delta_time)
+{
+	if(timer_on) {
+		time_left -= delta_time;
+		if(time_left<=0) {
+			visible=0;
+			timer_on=0;
+		}
+	}
+
+	//	printf("Time left: %d (%d)\n", time_left, timer_on);
+}
+
+// new timeout value
+void StdTransBtWin::set_timeout(int _time_out) 
+{
+	if(_time_out==0) timer_on = 0;
+	else timer_on = 1;
+
+	time_left = _time_out;
+}
+
 ///////////////////////////////////// Tab //////////////////////////////////////
 // Everything to handle tabs
 ////////////////////////////////////////////////////////////////////////////////
@@ -1073,11 +1442,12 @@ int CursorBar::onMove(int x, int y)
 IntIncDec::IntIncDec(const s_font* _font, const s_texture* tex_up,
 		const s_texture* tex_down, int _min, int _max,
 		int _init_value, int _inc) :
-	Container(), value(_init_value), min(_min), max(_max), inc(_inc), btmore(NULL), btless(NULL)
+	Container(), value(_init_value), min(_min), max(_max), inc(_inc), btmore(NULL), btless(NULL), label(NULL)
 {
-	if (_font) label.setFont(_font);
-	label.setSize(30,10);
-	label.setPos(9,2);
+	label = new Label;
+	if (_font) label->setFont(_font);
+	label->setSize(30,10);
+	label->setPos(9,2);
 	btmore = new TexturedButton(tex_up);
 	btmore->setSize(8,8);
 	btmore->setPos(0,0);
@@ -1090,14 +1460,12 @@ IntIncDec::IntIncDec(const s_font* _font, const s_texture* tex_up,
 	btless->setOnPressCallback(callback<void>(this, &IntIncDec::dec_value));
 	addComponent(btmore);
 	addComponent(btless);
-	addComponent(&label);
+	addComponent(label);
 	setSize(40,20);
 }
 
 IntIncDec::~IntIncDec()
 {
-	if (btmore) delete btmore;
-	if (btless) delete btless;
 }
 
 void IntIncDec::draw()
@@ -1105,7 +1473,7 @@ void IntIncDec::draw()
 	if (!visible) return;
 	ostringstream os;
 	os << value;
-	label.setLabel(os.str());
+	label->setLabel(os.str());
 	Container::draw();
 }
 
@@ -1114,7 +1482,7 @@ IntIncDecVert::IntIncDecVert(const s_font* _font, const s_texture* tex_up,
 		const s_texture* tex_down, int _min, int _max,
 		int _init_value, int _inc) : IntIncDec(_font, tex_up, tex_down, _min, _max, _init_value, _inc)
 {
-	label.setPos(0,3);
+	label->setPos(0,3);
 	btmore->setPos(_max/10 * 8 + 8, 0);
 	btless->setPos(_max/10 * 8 + 8, 8);
 	setSize(_max/10 * 8 + 16,40);
@@ -1123,11 +1491,12 @@ IntIncDecVert::IntIncDecVert(const s_font* _font, const s_texture* tex_up,
 FloatIncDec::FloatIncDec(const s_font* _font, const s_texture* tex_up,
 		const s_texture* tex_down, float _min, float _max,
 		float _init_value, float _inc) :
-	Container(), value(_init_value), min(_min), max(_max), inc(_inc), btmore(NULL), btless(NULL)
+	Container(), value(_init_value), min(_min), max(_max), inc(_inc), btmore(NULL), btless(NULL), label(NULL)
 {
-	if (_font) label.setFont(_font);
-	label.setSize(30,10);
-	label.setPos(9,2);
+	label = new Label;
+	if (_font) label->setFont(_font);
+	label->setSize(30,10);
+	label->setPos(9,2);
 	btless = new TexturedButton(tex_up);
 	btless->setSize(8,8);
 	btless->setPos(0,0);
@@ -1140,14 +1509,12 @@ FloatIncDec::FloatIncDec(const s_font* _font, const s_texture* tex_up,
 	btmore->setOnPressCallback(callback<void>(this, &FloatIncDec::dec_value));
 	addComponent(btmore);
 	addComponent(btless);
-	addComponent(&label);
+	addComponent(label);
 	setSize(50,20);
 }
 
 FloatIncDec::~FloatIncDec()
 {
-	if (btmore) delete btmore;
-	if (btless) delete btless;
 }
 
 void FloatIncDec::draw()
@@ -1155,7 +1522,7 @@ void FloatIncDec::draw()
 	if (!visible) return;
 	ostringstream os;
 	os << value;
-	label.setLabel(os.str());
+	label->setLabel(os.str());
 	Container::draw();
 }
 
@@ -1166,12 +1533,13 @@ Time_item::Time_item(const s_font* _font, const s_texture* tex_up,
 {
 	if (_font) setFont(_font);
 
+	// ranges are +1 and -1 from normal to allow rollover
 	y = new IntIncDec(getFont(), tex_up, tex_down, -9999, 99999, 1930, 1);
-	m = new IntIncDec(getFont(), tex_up, tex_down, 1, 12, 12, 1);
-	d = new IntIncDec(getFont(), tex_up, tex_down, 1, 31, 11, 1);
-	h = new IntIncDec(getFont(), tex_up, tex_down, 0, 23, 16, 1);
-	mn= new IntIncDec(getFont(), tex_up, tex_down, 0, 59, 35, 1);
-	s = new IntIncDec(getFont(), tex_up, tex_down, 0, 59, 23, 1);
+	m = new IntIncDec(getFont(), tex_up, tex_down, 0, 13, 12, 1);
+	d = new IntIncDec(getFont(), tex_up, tex_down, 0, 32, 11, 1);
+	h = new IntIncDec(getFont(), tex_up, tex_down, -1, 24, 16, 1);
+	mn= new IntIncDec(getFont(), tex_up, tex_down, -1, 60, 35, 1);
+	s = new IntIncDec(getFont(), tex_up, tex_down, -1, 60, 23, 1);
 
 	y->setOnPressCallback(callback<void>(this, &Time_item::onTimeChange));
 	m->setOnPressCallback(callback<void>(this, &Time_item::onTimeChange));
@@ -1246,6 +1614,20 @@ double Time_item::getJDay(void) const
     return floor(365.25 * iy) + floor(30.6001 * (im + 1)) + B + 1720996.5 + id + ih / 24.0 + imn / 1440.0 + (double)is / 86400.0;
 }
 
+// for use with commands - no special characters, just the local date
+string Time_item::getDateString(void)
+{
+	ostringstream os;
+	os << y->getValue() << ":"
+	   << m->getValue() << ":"
+	   << d->getValue() << "T"
+	   << h->getValue() << ":"
+	   << mn->getValue() << ":"
+	   << s->getValue();
+	return os.str();
+}
+
+
 void Time_item::setJDay(double JD)
 {
 	static int iy, im, id, ih, imn, is;
