@@ -1,6 +1,6 @@
 /*
  * Stellarium
- * Copyright (C) 2002 Fabien Chreau
+ * Copyright (C) 2002 Fabien Chereau
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,6 +45,9 @@
 using namespace std;
 using namespace boost;
 
+// Tony - Editbox blink callback (can't get to work with the class!)
+Uint32 toggleBlink(Uint32 interval);
+
 namespace s_gui
 {
 	// gui Return Values:
@@ -53,6 +56,8 @@ namespace s_gui
 		S_GUI_MOUSE_LEFT,
 		S_GUI_MOUSE_RIGHT,
     	S_GUI_MOUSE_MIDDLE,
+		S_GUI_MOUSE_WHEELUP,
+		S_GUI_MOUSE_WHEELDOWN,
 		S_GUI_PRESSED,
 		S_GUI_RELEASED
 	};
@@ -82,6 +87,7 @@ namespace s_gui
     {
     public:
         Painter();
+		~Painter();
 		Painter(const s_texture* _tex1, const s_font* _font, const s_color& _baseColor, const s_color& _textColor);
 		void drawSquareEdge(const s_vec2i& pos, const s_vec2i& sz) const;
 		void drawSquareEdge(const s_vec2i& pos, const s_vec2i& sz, const s_color& c) const;
@@ -138,7 +144,7 @@ namespace s_gui
         virtual int getFocus(void) const {return focus;}
 		virtual int onClic(int, int, S_GUI_VALUE, S_GUI_VALUE) {return 0;}
 		virtual int onMove(int, int) {return 0;}
-		virtual int onKey(SDLKey, S_GUI_VALUE) {return 0;}
+		virtual int onKey(Uint16, S_GUI_VALUE) {return 0;}
 		virtual void setTexture(const s_texture* tex) {painter.setTexture(tex);}
 		virtual void setFont(const s_font* f) {painter.setFont(f);}
 		const s_font* getFont(void) const {return painter.getFont();}
@@ -148,6 +154,7 @@ namespace s_gui
 		virtual int isIn(int x, int y);
 		static void setDefaultPainter(const Painter& p) {defaultPainter=p;}
 		static void initScissor(int winW, int winH);
+		static void deleteScissor(void);
 		static void enableScissor(void) {scissor->activate();}
 		static void disableScissor(void) {scissor->desactivate();}
     protected:
@@ -188,7 +195,7 @@ namespace s_gui
         virtual void draw(void);
 		virtual int onClic(int, int, S_GUI_VALUE, S_GUI_VALUE);
 		virtual int onMove(int, int);
-		virtual int onKey(SDLKey, S_GUI_VALUE);
+		virtual int onKey(Uint16, S_GUI_VALUE);
     protected:
         std::list<Component*> childs;
     };
@@ -255,6 +262,7 @@ namespace s_gui
         virtual const string& getLabel() const {return label;}
         virtual void setLabel(const string&);
         virtual void draw();
+        virtual void draw(float intensity);
 		virtual void adjustSize(void);
     protected:
         string label;
@@ -274,6 +282,57 @@ namespace s_gui
 		Label label;
     };
 
+    // Tony - Editbox
+    class EditBox : public Button
+    {
+    public:
+        EditBox(const string& _label = "", const s_font* font = NULL);
+		virtual ~EditBox();
+		virtual void setOnReturnKeyCallback(const callback<void>& c) {onReturnKeyCallback = c;}
+		virtual void setOnAutoCompleteCallback(const callback<void>& c) {onAutoCompleteCallback = c;}
+        virtual void draw(void);
+		virtual void setActive(int _active) {Button::setActive(_active); label.setActive(_active);}
+		virtual void setFont(const s_font* f) {Button::setFont(f); label.setFont(f);}
+		virtual void setTextColor(const s_color& c) {Button::setTextColor(c); label.setTextColor(c);}
+		virtual void setPainter(const Painter& p) {Button::setPainter(p); label.setPainter(p);}
+		virtual int onKey(Uint16, S_GUI_VALUE);
+		virtual int onClic(int, int, S_GUI_VALUE, S_GUI_VALUE);
+		virtual void setAutoComplete(vector<string> _autocomplete) { lstAutoComplete = _autocomplete; };
+		string getAutoCompleteOptions(void) { return autoCompleteOptions; }
+		string getText(void) { return text; }
+		void setFocus(void);
+		void resetFocus(void);
+		void refreshLabel(void);
+	protected:
+		callback<void> onReturnKeyCallback;
+		callback<void> onAutoCompleteCallback;
+		callback<void> onWordCountChangedCallback;
+		vector<string> lstAutoComplete;
+        string autoCompleteOptions;
+        int countAutoCompleteOptions;
+        int countAutoCompletePos;
+        string firstAutoComplete;
+        bool autoCompleteReady;
+		void testAutoComplete(void);
+		void resetAutoComplete(bool keepChanges);
+
+		void clearText(void);
+
+		void addHistory(const string& _history);
+		string prevHistory(void);
+		string nextHistory(void);
+		void resetHistory(void);
+		string history[10];
+        int historyPos, historyMaxPos;
+        
+        void cursorToNextWord(void);
+        void cursorToPrevWord(void);
+
+        Label label;
+		bool isEditing;
+		string text;
+		unsigned int cursorPos;
+    };
 
     class TextLabel : public Container
 	{
@@ -304,7 +363,7 @@ namespace s_gui
         int value, min, max, inc;
 		TexturedButton* btmore;
 		TexturedButton* btless;
-		Label label;
+		Label * label;
 	};
 
 	class IntIncDecVert : public IntIncDec
@@ -331,7 +390,7 @@ namespace s_gui
         float value, min, max, inc;
 		TexturedButton* btmore;
 		TexturedButton* btless;
-		Label label;
+		Label * label;
 	};
 
 	// Widget used to set time and date.
@@ -341,6 +400,7 @@ namespace s_gui
 		Time_item(const s_font* _font = NULL, const s_texture* tex_up = NULL,
 			const s_texture* tex_down = NULL, double _JD = 2451545.0);
 		double getJDay(void) const;
+		string getDateString(void);
 		void setJDay(double jd);
 		virtual void draw();
 		void onTimeChange(void);
@@ -411,6 +471,19 @@ namespace s_gui
 		callback<void> onHideBtCallback;
 	};
 
+	// stdbtwin with transient (timeout) functionality
+	class StdTransBtWin : public StdBtWin
+	{
+	public:
+	    StdTransBtWin(const char * _title = NULL, int _time_out =0, s_texture* _header_tex = NULL,
+			s_font * _winfont = NULL, int headerSize = 18);
+		virtual void update(int _delta_time);
+		virtual void set_timeout(int _time_out=0);
+	protected:
+		int time_left;
+		bool timer_on;
+	};
+
 	class TabHeader : public LabeledButton
 	{
 	public:
@@ -497,6 +570,15 @@ namespace s_gui
 		virtual void draw(void);
 		virtual int onClic(int, int, S_GUI_VALUE, S_GUI_VALUE);
 		void addItem(const string &);
+		void addItemList(const string& s)
+		{
+			istringstream is(s);
+			string elem;
+			while(getline(is, elem))
+			{
+				addItem(elem);
+			}
+		}
 		const string getValue() const;
 		bool setValue(const string &);
 		void adjustSize(void) {setSizey(elemsSize);}
