@@ -1,6 +1,6 @@
 /*
 * Stellarium
-* Copyright (C) 2003 Fabien Chreau
+* Copyright (C) 2003 Fabien Chereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
@@ -89,7 +89,7 @@ string Container::getString(void)
 }
 
 
-bool Container::onKey(SDLKey k, S_TUI_VALUE s)
+bool Container::onKey(Uint16 k, S_TUI_VALUE s)
 {
 	list<Component*>::iterator iter = childs.begin();
 	while (iter != childs.end())
@@ -149,7 +149,7 @@ bool Branch::setValue_Specialslash(const string& s)
 	return false;
 }
 
-bool Branch::onKey(SDLKey k, S_TUI_VALUE v)
+bool Branch::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (!*current) return false;
 	if (v==S_TUI_RELEASED) return (*current)->onKey(k,v);
@@ -180,7 +180,7 @@ MenuBranch::MenuBranch(const string& s) : Branch(), label(s), isNavigating(false
 {
 }
 
-bool MenuBranch::onKey(SDLKey k, S_TUI_VALUE v)
+bool MenuBranch::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (isNavigating)
 	{
@@ -246,7 +246,7 @@ MenuBranch_item::MenuBranch_item(const string& s) : Branch(), label(s), isEditin
 {
 }
 
-bool MenuBranch_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool MenuBranch_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (isEditing)
 	{
@@ -302,7 +302,7 @@ Boolean_item::Boolean_item(bool init_state, const string& _label, const string&
 	string_disabled = _string_disabled;
 }
 
-bool Boolean_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Boolean_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_PRESSED && (k==SDLK_UP || k==SDLK_DOWN) )
 	{
@@ -334,7 +334,7 @@ string Decimal::getString(void)
 	return os.str();
 }
 
-bool Integer_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Integer_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_RELEASED) return false;
 	if (!numInput)
@@ -443,7 +443,7 @@ string Integer_item::getString(void)
 	return os.str();
 }
 
-bool Decimal_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Decimal_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_RELEASED) return false;
 	if (!numInput)
@@ -562,26 +562,28 @@ Time_item::Time_item(const string& _label, double _JD) :
 	CallbackComponent(), JD(_JD), current_edit(NULL), label(_label),
 	y(NULL), m(NULL), d(NULL), h(NULL), mn(NULL), s(NULL)
 {
+	// Note: range limits are 1 beyond normal range to allow for rollover
+	// as date is calculated after updates and range limits are enforced then
 	y = new Integer_item(-100000, 100000, 2003);
-	m = new Integer_item(1, 12, 1);
-	d = new Integer_item(1, 31, 1);
-	h = new Integer_item(0, 23, 0);
-	mn = new Integer_item(0, 59, 0);
-	s = new Integer_item(0, 59, 0);
+	m = new Integer_item(0, 13, 1);
+	d = new Integer_item(0, 32, 1);
+	h = new Integer_item(-1, 24, 0);
+	mn = new Integer_item(-1, 60, 0);
+	s = new Integer_item(-1, 60, 0);
 	current_edit = y;
 }
 
 Time_item::~Time_item()
 {
-	if (y) delete y;
-	if (m) delete y;
-	if (d) delete y;
-	if (h) delete y;
-	if (mn) delete y;
-	if (s) delete y;
+	delete y;
+	delete m;
+	delete d;
+	delete h;
+	delete mn;
+	delete s;
 }
 
-bool Time_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Time_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_RELEASED) return false;
 
@@ -649,6 +651,23 @@ string Time_item::getString(void)
 	return os.str();
 }
 
+
+// for use with commands - no special characters, just the local date
+string Time_item::getDateString(void)
+{
+	compute_ymdhms();  // possibly redundant
+
+	ostringstream os;
+	os << y->getString() << ":"
+	   << m->getString() << ":"
+	   << d->getString() << "T"
+	   << h->getString() << ":"
+	   << mn->getString() << ":"
+	   << s->getString();
+	return os.str();
+}
+
+
 // Code originally from libnova which appeared to be totally wrong... New code from celestia
 void Time_item::compute_ymdhms(void)
 {
@@ -758,7 +777,7 @@ Time_zone_item::Time_zone_item(const string& zonetab_file, const string& _label)
 	current_edit=&continents_names;
 }
 
-bool Time_zone_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Time_zone_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_RELEASED) return false;
 
@@ -836,7 +855,7 @@ string Action_item::getString(void)
 	}
 }
 
-bool Action_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool Action_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_PRESSED && k==SDLK_RETURN)
 	{
@@ -864,7 +883,7 @@ string ActionConfirm_item::getString(void)
 	else return label + string_prompt1;
 }
 
-bool ActionConfirm_item::onKey(SDLKey k, S_TUI_VALUE v)
+bool ActionConfirm_item::onKey(Uint16 k, S_TUI_VALUE v)
 {
 	if (v==S_TUI_PRESSED && k==SDLK_RETURN)
 	{
