@@ -1,6 +1,6 @@
 /*
  * Stellarium
- * Copyright (C) 2002 Fabien Chreau
+ * Copyright (C) 2002 Fabien Chereau
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,7 +40,17 @@
 #define MY_MAX(a,b) (((a)>(b))?(a):(b))
 #define MY_MIN(a,b) (((a)<(b))?(a):(b))
 
-// needed for bug workarounds
-// #define NVIDIA TRUE
+// Used for GNU gettext translations
+#ifndef MACOSX
+#include "gettext.h"
+#define _(String) gettext (String)
+#define N_(String) gettext_noop(String)
+#else
+# include "POSupport.h"
+# define _(String) localizedUTF8String(String)
+# define N_(String) (String)
+#endif
+
+#include <cassert>
 
 #endif /*_STELLARIUM_H_*/
